ClassDescription subclass:#JavaClass
	instanceVariableNames:'fullName accessFlags constantPool sourceFile fields initialized'
	classVariableNames:''
	poolDictionaries:''
	category:'Java-Classes'
!


!JavaClass class methodsFor:'instance creation'!

fullName:aString
    ^ self new setFullName:aString

    "Created: 15.4.1996 / 15:52:55 / cg"
!

fullName:aString numStatic:nStatic
    |meta cls|

    "create the metaclass first"

    meta := Metaclass new.
    meta setSuperclass:(self).
    meta instSize:(JavaClass class instSize + nStatic).
    meta setName:(aString , 'class') asSymbol.
    meta setClassVariableString:''.

    "then let the new meta create the class"
    cls := meta new.
    cls setSuperclass:JavaObject.
    cls instSize:0.
    cls setName:aString.
    cls flags:0.
"/    cls setClassVariableString:''.
    cls setInstanceVariableString:''.
    cls category:#java.

    cls setFullName:aString.
    ^ cls

"/    ^ self new setFullName:aString

    "Created: 15.4.1996 / 15:52:55 / cg"
!

name:aString
    self shouldNotImplement

    "Created: 15.4.1996 / 15:52:55 / cg"
! !

!JavaClass class methodsFor:'special'!

setInstanceVariableStringFromFields:f in:aClass
    |varNames|

    varNames := ''.
    f do:[:aField |
        varNames := varNames , aField name , ' '
    ].
    aClass setInstanceVariableString:varNames.

    "Created: 15.4.1996 / 16:42:52 / cg"
! !

!JavaClass methodsFor:'accessing'!

category
    "java classes do not really hav a category;
     simulate one by separating its name into individual components,
     prepending 'java'"

    |nameComponents|

    nameComponents := fullName asCollectionOfSubstringsSeparatedBy:$/.
    ^ 'java-' , (nameComponents asStringWith:$/ from:1 to:(nameComponents size - 1))
!

constantPool
    ^ constantPool
!

fullName
    ^ fullName
!

name
    ^ name "/ fullName
!

package
    "extract from the fullName"

    |components|

    components := fullName asCollectionOfSubstringsSeparatedBy:$/.
    components size > 1 ifTrue:[
        ^ (components copyWithoutLast:1) asStringWith:$/
    ].
    ^ fullName

    "
     Java allClasses first fullName
     Java allClasses first package  
    "
!

sourceFile
    ^ sourceFile
! !

!JavaClass methodsFor:'java initialization'!

classInit
    "call the classes JAVA clinit function"

    |idx|

    initialized := true.

    (idx := self selectorArray indexOf:#'<clinit>()V') ~~ 0 ifTrue:[
        'calling clinit() of ' print. self fullName printNL.
        [
            self invokeJavaMethod:(self methodArray at:idx).
        ] valueOnUnwindDo:[
            initialized := false
        ]
    ] ifFalse:[
        self fullName print. ' has no clinit()' printNL.
    ].

    "
     JavaInterpreter instructionTrace:true.
     JavaInterpreter callTrace:true.

     (Java classNamed:'java.lang.String') classInit
     (Java classNamed:'java.lang.System') classInit

     (Java classNamed:'java/io/FileDescriptor') classInit
     (Java classNamed:'java.util.Properties') classInit 
    "
! !

!JavaClass methodsFor:'java instance creation'!

basicNew
    "create a new cleared JAVA instance"

    |newJavaObject|

    newJavaObject := super basicNew.

    "/
    "/ this is WRONG:
    "/ must look for fields and initialize as apropriate
    "/ for its type:
    "/       int -> 0
    "/    object -> nil
    "/     reals -> 0.0
    "/     chars -> 0
    "/
    "/ for now, fill them with 0, depending on JAVA init-code to
    "/ initialize fields soon ...
    "/ ... we will see.
    "/
    newJavaObject initializeToZero.     "/ mhmh
    ^ newJavaObject

    "
     (Java classNamed:'java.lang.String') basicNew inspect
     (Java classNamed:'java.lang.String') new inspect
    "
!

new
    "create a new instance, and call its JAVA init function"

    |newJavaObject|

    newJavaObject := self basicNew.
    newJavaObject invoke:#'<init>'.
    ^ newJavaObject

    "
     (Java classNamed:'java.lang.String') basicNew inspect
     (Java classNamed:'java.lang.String') new inspect
    "


! !

!JavaClass methodsFor:'message sending'!

invokeJava:selector
    "send javaSelector (name+sig) message, without arguments"

    |method|

    method := self lookupMethodFor:selector.
    method notNil ifTrue:[
        ^ self invokeJavaMethod:method
    ].

    ^ self doesNotUnderstand:(Message selector:selector)


!

invokeJava:selector with:arg
    "send javaSelector (name+sig) message, with 1 argument"

    |method|

    method := self lookupMethodFor:selector.
    method notNil ifTrue:[
        ^ self invokeJavaMethod:method with:arg
    ].
    ^ self doesNotUnderstand:(Message selector:selector)

!

invokeJavaMethod:aJavaMethod
    "invoke a static java method, without arguments"

    |i val|

    aJavaMethod numArgs ~~ 0 ifTrue:[
        self halt:'need arguments'
    ].
    aJavaMethod isStatic ifFalse:[
        self halt:'non-static function'
    ].

    i := JavaInterpreter new.
    val := i interpret:aJavaMethod sender:thisContext sender.

    ^ JavaObject convertJavaObject:val signature:(aJavaMethod retValSignature)


!

invokeJavaMethod:aJavaMethod with:arg
    "invoke a static java method, with one argument"

    |i val|

    aJavaMethod numArgs ~~ 1 ifTrue:[
        self halt:'argument count'
    ].
    aJavaMethod isStatic ifFalse:[
        self halt:'non-static function'
    ].

    i := JavaInterpreter new.
    i push:arg.

    val := i interpret:aJavaMethod sender:thisContext sender.

    ^ JavaObject convertJavaObject:val signature:(aJavaMethod retValSignature)


!

lookupMethodFor:selector
    |method cls sel|

    sel := selector asSymbolIfInterned.
    sel notNil ifTrue:[
        cls := self.
        [cls notNil] whileTrue:[
            method := cls compiledMethodAt:sel.
            method notNil ifTrue:[ ^ method ].
            cls := cls superclass.
        ].
    ].

    ^ nil
! !

!JavaClass methodsFor:'printing & storing'!

displayString
    ^ fullName copy replaceAll:$/ by:$.
"/    ^ 'JAVA-' , name .
"/    ^ name , '(Java)'  "/ 'JavaClass(' , name , ')'

    "Created: 15.4.1996 / 16:02:48 / cg"
! !

!JavaClass methodsFor:'private accessing'!

addMethod:m name:name signature:signature
    selectorArray isNil ifTrue:[
        selectorArray := #().
        methodArray := #()
    ].
    selectorArray := selectorArray copyWith:(name , signature) asSymbol. "/ name.
"/    selectorArray := selectorArray copyWith:name asSymbol. "/ name.
    methodArray := methodArray copyWith:m

    "Created: 15.4.1996 / 16:42:52 / cg"
!

markUninitialized
    initialized := false.
!

setAccessFlags:flags
    accessFlags := flags.

    "Created: 15.4.1996 / 16:42:52 / cg"
!

setConstantPool:anArray
    constantPool := anArray.

    "Created: 15.4.1996 / 16:42:52 / cg"
!

setFields:f
    JavaClass setInstanceVariableStringFromFields:f in:self.

    "Created: 15.4.1996 / 16:42:52 / cg"
!

setFullName:aString
    |nameComponents|

    fullName := aString.
    nameComponents := aString asCollectionOfSubstringsSeparatedBy:$/.
    name := nameComponents last

    "Created: 15.4.1996 / 16:42:52 / cg"
!

setSourceFile:aFilename
    sourceFile := aFilename.

    "Created: 15.4.1996 / 16:42:52 / cg"
!

setSuperclass:aClass
    aClass isNil ifTrue:[
        super setSuperclass:JavaObject
    ] ifFalse:[
        (aClass isMemberOf:JavaUnresolvedClassConstant) ifTrue:[
            self halt.
        ].
        super setSuperclass:aClass.
    ].

    instvars notNil ifTrue:[
        (superclass notNil and:[(superclass isMemberOf:JavaUnresolvedClassConstant) not]) ifTrue:[
            instSize := superclass instSize + (instvars asCollectionOfWords size)
        ] ifFalse:[
            instSize := instvars asCollectionOfWords size
        ]
    ].

    "Created: 15.4.1996 / 16:42:52 / cg"
! !

!JavaClass methodsFor:'queries'!

browserClass
    "return the browser to use for this class - 
     this can be redefined in special classes, to get different browsers"

    ^ JavaBrowser




!

isClass
    ^ true
!

isFinal
    ^ (accessFlags bitAnd:16r0010) ~~ 0
!

isInitialized
    ^ initialized == true
!

isInterface
    ^ (accessFlags bitAnd:16r0200) ~~ 0
!

isPublic
    ^ (accessFlags bitAnd:16r0001) ~~ 0
! !

!JavaClass methodsFor:'resolving class refs'!

resolveClassRefs
    |newClass const newConst|

    (superclass isMemberOf:JavaUnresolvedClassConstant) ifTrue:[
        newClass := superclass resolve.
        newClass isNil ifTrue:[
            newClass := JavaClassReader loadClassLazy:(superclass fullName).
        ].
        newClass notNil ifTrue:[
            superclass := newClass
        ]
    ].

    1 to:constantPool size do:[:index |
        const := constantPool at:index.
        (const isMemberOf:JavaUnresolvedClassConstant) ifTrue:[
            "/ try to resolve
            newConst := const resolve.
            newConst isNil ifTrue:[
                newConst := JavaClassReader loadClassLazy:(const fullName).
            ].
            newConst notNil ifTrue:[
                constantPool at:index put:newConst
            ]
        ]
    ].

    (superclass notNil and:[(superclass isMemberOf:JavaUnresolvedClassConstant) not]) ifTrue:[
        superclass resolveClassRefs.
        instSize := superclass instSize + (instvars asCollectionOfWords size)
    ] ifFalse:[
        instSize := instvars asCollectionOfWords size
    ].
"/Transcript show:'resolved '; show: self displayString; 
"/           show: ' instSize='; showCr:  instSize.
! !

!JavaClass methodsFor:'smalltalk interface'!

invoke:selector
    "send a message, without args"

    |method cls sel|

    sel := selector asSymbolIfInterned.
    sel notNil ifTrue:[
        cls := self.
        [cls notNil and:[cls ~~ JavaObject]] whileTrue:[
            cls methodArray do:[:aMethod |
"/     aMethod name printNL.
                aMethod name == selector ifTrue:[
                    aMethod numArgs == 0 ifTrue:[
                        ^ self invokeJavaMethod:aMethod
                    ]
                ]
            ].
            cls := cls superclass.
        ].
    ].

    ^ self doesNotUnderstand:(Message selector:selector)

    "
     |stack|

     stack := (Java at:'java/util/Stack') basicNew.
     stack invoke:#'<init>'. 
    "
    "
     |stack|

     stack := (Java at:'java/util/Stack') basicNew.
     stack invoke:#isEmpty. 
    "
    "
     |stack|

     stack := (Java at:'java/util/Stack') basicNew.
     stack invoke:#size. 
    "


! !

!JavaClass class methodsFor:'documentation'!

version
    ^ '$Header: /home/jv/Projects/SmalltalkX/repositories/cvs/stx/libjava/JavaClass.st,v 1.17 1996/05/03 18:55:54 cg Exp $'
! !
